#XfceNotifyWindow {
    background-color: #ff69b4;
    border-radius: 10px;
    border-color : #ffff00;
    border-style: outset;
    border-width: 6px;
}

#XfceNotifyWindow GtkLabel,
#XfceNotifyWindow GtkImage {
    font : Comic Sans MS 22;
    color: #ffff00;
    -GtkWidget-link-color:#c17800;
}

#XfceNotifyWindow GtkLabel#summary {
    font : Bold;
}

#XfceNotifyWindow GtkButton {
    border-image: none;
    background-image: none;
    background-color : #9400d3;
}

#XfceNotifyWindow GtkButton:hover {
    border-image: none;
    background-image: none;
    background-color : #5e0086;
}

#XfceNotifyWindow GtkProgressBar {
    border-radius: 3px;
}

#XfceNotifyWindow GtkProgressBar .progressbar {
    min-height: 16px;
    background-image: none;
    background-color: shade(#ffff00, 1.2);
}

#XfceNotifyWindow GtkProgressBar .trough {
    min-height: 16px;
    background-image: none;
    background-color: #5e0086;
    border: 1px solid shade(#5e0086, 0.6);
    border-radius: 3px;
}
