


#ifndef __XFCE_NOTIFY_ENUM_TYPES_H__
#define __XFCE_NOTIFY_ENUM_TYPES_H__
#include <glib-object.h>
G_BEGIN_DECLS
/* enumerations from "./xfce4-notifyd/xfce-notify-window.h" */
GType xfce_notify_close_reason_get_type(void) G_GNUC_CONST;
#define XFCE_TYPE_NOTIFY_CLOSE_REASON (xfce_notify_close_reason_get_type())
G_END_DECLS

#endif /* __XFCE_NOTIFY_ENUM_TYPES_H__ */



