/* automatically generated from xfce4-notifyd-config/xfce4-notifyd-config.glade */
#ifdef __SUNPRO_C
#pragma align 4 (xfce4_notifyd_config_ui)
#endif
#ifdef __GNUC__
static const char xfce4_notifyd_config_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char xfce4_notifyd_config_ui[] =
#endif
{
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!-- Generated with glade 3"
  ".20.0 -->\n<interface>\n  <requires lib=\"gtk+\" version=\"3.14\"/>\n  "
  "<requires lib=\"libxfce4ui-2\" version=\"4.12\"/>\n  <object class=\"Gt"
  "kAdjustment\" id=\"adjustment1\">\n    <property name=\"lower\">1</prop"
  "erty>\n    <property name=\"upper\">480</property>\n    <property name="
  "\"value\">10</property>\n    <property name=\"step_increment\">1</prope"
  "rty>\n    <property name=\"page_increment\">10</property>\n  </object>\n"
  "  <object class=\"GtkAdjustment\" id=\"adjustment2\">\n    <property na"
  "me=\"upper\">1.01</property>\n    <property name=\"value\">0.9000000000"
  "0000002</property>\n    <property name=\"step_increment\">0.01</propert"
  "y>\n    <property name=\"page_increment\">0.10000000000000001</property"
  ">\n    <property name=\"page_size\">0.01</property>\n  </object>\n  <ob"
  "ject class=\"GtkListStore\" id=\"liststore1\">\n    <columns>\n      <!"
  "-- column-name gchararray -->\n      <column type=\"gchararray\"/>\n   "
  " </columns>\n    <data>\n      <row>\n        <col id=\"0\" translatabl"
  "e=\"yes\">only during \"Do not disturb\"</col>\n      </row>\n      <ro"
  "w>\n        <col id=\"0\" translatable=\"yes\">always</col>\n      </ro"
  "w>\n    </data>\n  </object>\n  <object class=\"GtkListStore\" id=\"lis"
  "tstore2\">\n    <columns>\n      <!-- column-name gchararray -->\n     "
  " <column type=\"gchararray\"/>\n    </columns>\n    <data>\n      <row>"
  "\n        <col id=\"0\" translatable=\"yes\">all</col>\n      </row>\n "
  "     <row>\n        <col id=\"0\" translatable=\"yes\">all except block"
  "ed</col>\n      </row>\n      <row>\n        <col id=\"0\" translatable"
  "=\"yes\">only blocked</col>\n      </row>\n    </data>\n  </object>\n  "
  "<object class=\"GtkListStore\" id=\"liststore3\">\n    <columns>\n     "
  " <!-- column-name gchararray -->\n      <column type=\"gchararray\"/>\n"
  "    </columns>\n    <data>\n      <row>\n        <col id=\"0\" translat"
  "able=\"yes\">display with mouse pointer</col>\n      </row>\n      <row"
  ">\n        <col id=\"0\" translatable=\"yes\">primary display</col>\n  "
  "    </row>\n    </data>\n  </object>\n  <object class=\"GtkListStore\" "
  "id=\"model1\">\n    <columns>\n      <!-- column-name gchararray -->\n "
  "     <column type=\"gchararray\"/>\n    </columns>\n    <data>\n      <"
  "row>\n        <col id=\"0\" translatable=\"yes\">Top left</col>\n      "
  "</row>\n      <row>\n        <col id=\"0\" translatable=\"yes\">Bottom "
  "left</col>\n      </row>\n      <row>\n        <col id=\"0\" translatab"
  "le=\"yes\">Top right</col>\n      </row>\n      <row>\n        <col id="
  "\"0\" translatable=\"yes\">Bottom right</col>\n      </row>\n    </data"
  ">\n  </object>\n  <object class=\"XfceTitledDialog\" id=\"notifyd_setti"
  "ngs_dlg\">\n    <property name=\"can_focus\">False</property>\n    <pro"
  "perty name=\"title\" translatable=\"yes\">Notifications</property>\n   "
  " <property name=\"window_position\">center-on-parent</property>\n    <p"
  "roperty name=\"default_width\">550</property>\n    <property name=\"def"
  "ault_height\">600</property>\n    <property name=\"icon_name\">xfce4-no"
  "tifyd</property>\n    <property name=\"type_hint\">dialog</property>\n "
  "   <property name=\"subtitle\" translatable=\"yes\">Customize your noti"
  "fication settings</property>\n    <child internal-child=\"vbox\">\n    "
  "  <object class=\"GtkBox\" id=\"dialog-vbox1\">\n        <property name"
  "=\"visible\">True</property>\n        <property name=\"can_focus\">Fals"
  "e</property>\n        <property name=\"orientation\">vertical</property"
  ">\n        <property name=\"spacing\">2</property>\n        <child inte"
  "rnal-child=\"action_area\">\n          <object class=\"GtkButtonBox\" i"
  "d=\"dialog-action_area1\">\n            <property name=\"visible\">True"
  "</property>\n            <property name=\"can_focus\">False</property>\n"
  "            <property name=\"hexpand\">True</property>\n            <pr"
  "operty name=\"homogeneous\">True</property>\n            <child>\n     "
  "         <object class=\"GtkButton\" id=\"help_btn\">\n                "
  "<property name=\"label\">gtk-help</property>\n                <property"
  " name=\"visible\">True</property>\n                <property name=\"can"
  "_focus\">True</property>\n                <property name=\"receives_def"
  "ault\">True</property>\n                <property name=\"use_stock\">Tr"
  "ue</property>\n              </object>\n              <packing>\n      "
  "          <property name=\"expand\">False</property>\n                <"
  "property name=\"fill\">False</property>\n                <property name"
  "=\"position\">0</property>\n                <property name=\"secondary\""
  ">True</property>\n              </packing>\n            </child>\n     "
  "       <child>\n              <object class=\"GtkButton\" id=\"close_bt"
  "n\">\n                <property name=\"label\">gtk-close</property>\n  "
  "              <property name=\"use_action_appearance\">False</property>"
  "\n                <property name=\"visible\">True</property>\n         "
  "       <property name=\"can_focus\">True</property>\n                <p"
  "roperty name=\"receives_default\">True</property>\n                <pro"
  "perty name=\"use_stock\">True</property>\n              </object>\n    "
  "          <packing>\n                <property name=\"expand\">True</pr"
  "operty>\n                <property name=\"fill\">True</property>\n     "
  "           <property name=\"pack_type\">end</property>\n               "
  " <property name=\"position\">1</property>\n              </packing>\n  "
  "          </child>\n          </object>\n          <packing>\n         "
  "   <property name=\"expand\">True</property>\n            <property nam"
  "e=\"fill\">True</property>\n            <property name=\"position\">0</"
  "property>\n          </packing>\n        </child>\n        <child>\n   "
  "       <object class=\"GtkBox\" id=\"plug-child\">\n            <proper"
  "ty name=\"visible\">True</property>\n            <property name=\"can_f"
  "ocus\">False</property>\n            <property name=\"border_width\">6<"
  "/property>\n            <property name=\"orientation\">vertical</proper"
  "ty>\n            <child>\n              <object class=\"GtkRevealer\" i"
  "d=\"do_not_disturb_info\">\n                <property name=\"visible\">"
  "True</property>\n                <property name=\"can_focus\">False</pr"
  "operty>\n                <property name=\"margin_bottom\">12</property>"
  "\n                <property name=\"reveal_child\">True</property>\n    "
  "            <child>\n                  <object class=\"GtkInfoBar\" id="
  "\"infobar1\">\n                    <property name=\"visible\">True</pro"
  "perty>\n                    <property name=\"app_paintable\">True</prop"
  "erty>\n                    <property name=\"can_focus\">False</property"
  ">\n                    <child internal-child=\"action_area\">\n        "
  "              <object class=\"GtkButtonBox\" id=\"infobar-action_area\""
  ">\n                        <property name=\"can_focus\">False</property"
  ">\n                        <property name=\"spacing\">6</property>\n   "
  "                     <property name=\"layout_style\">start</property>\n"
  "                        <child>\n                          <placeholder"
  "/>\n                        </child>\n                      </object>\n"
  "                      <packing>\n                        <property name"
  "=\"expand\">True</property>\n                        <property name=\"f"
  "ill\">True</property>\n                        <property name=\"positio"
  "n\">0</property>\n                      </packing>\n                   "
  " </child>\n                    <child internal-child=\"content_area\">\n"
  "                      <object class=\"GtkBox\" id=\"infobar-content_are"
  "a\">\n                        <property name=\"name\">8</property>\n   "
  "                     <property name=\"can_focus\">False</property>\n   "
  "                     <child>\n                          <object class=\""
  "GtkBox\" id=\"box1\">\n                            <property name=\"vis"
  "ible\">True</property>\n                            <property name=\"ca"
  "n_focus\">False</property>\n                            <property name="
  "\"spacing\">18</property>\n                            <child>\n       "
  "                       <object class=\"GtkImage\" id=\"image1\">\n     "
  "                           <property name=\"visible\">True</property>\n"
  "                                <property name=\"can_focus\">False</pro"
  "perty>\n                                <property name=\"icon_name\">di"
  "alog-information-symbolic</property>\n                                <"
  "property name=\"icon_size\">1</property>\n                             "
  " </object>\n                              <packing>\n                  "
  "              <property name=\"expand\">False</property>\n             "
  "                   <property name=\"fill\">True</property>\n           "
  "                     <property name=\"position\">0</property>\n        "
  "                      </packing>\n                            </child>\n"
  "                            <child>\n                              <obj"
  "ect class=\"GtkLabel\" id=\"infobar_label\">\n                         "
  "       <property name=\"visible\">True</property>\n                    "
  "            <property name=\"can_focus\">False</property>\n            "
  "                    <property name=\"label\" translatable=\"yes\">&lt;b"
  "&gt;Currently only urgent notifications are shown.&lt;/b&gt;</property>"
  "\n                                <property name=\"use_markup\">True</p"
  "roperty>\n                                <property name=\"ellipsize\">"
  "end</property>\n                                <property name=\"xalign"
  "\">0</property>\n                              </object>\n             "
  "                 <packing>\n                                <property n"
  "ame=\"expand\">True</property>\n                                <proper"
  "ty name=\"fill\">True</property>\n                                <prop"
  "erty name=\"position\">1</property>\n                              </pa"
  "cking>\n                            </child>\n                         "
  " </object>\n                          <packing>\n                      "
  "      <property name=\"expand\">False</property>\n                     "
  "       <property name=\"fill\">True</property>\n                       "
  "     <property name=\"position\">0</property>\n                        "
  "  </packing>\n                        </child>\n                      <"
  "/object>\n                      <packing>\n                        <pro"
  "perty name=\"expand\">False</property>\n                        <proper"
  "ty name=\"fill\">True</property>\n                        <property nam"
  "e=\"position\">0</property>\n                      </packing>\n        "
  "            </child>\n                    <child>\n                    "
  "  <placeholder/>\n                    </child>\n                  </obj"
  "ect>\n                </child>\n              </object>\n              "
  "<packing>\n                <property name=\"expand\">False</property>\n"
  "                <property name=\"fill\">True</property>\n              "
  "  <property name=\"position\">0</property>\n              </packing>\n "
  "           </child>\n            <child>\n              <object class=\""
  "GtkStackSwitcher\" id=\"stack_switcher\">\n                <property na"
  "me=\"visible\">True</property>\n                <property name=\"can_fo"
  "cus\">False</property>\n                <property name=\"halign\">cente"
  "r</property>\n                <property name=\"margin_left\">12</proper"
  "ty>\n                <property name=\"margin_right\">12</property>\n   "
  "             <property name=\"stack\">stack</property>\n              <"
  "/object>\n              <packing>\n                <property name=\"exp"
  "and\">False</property>\n                <property name=\"fill\">False</"
  "property>\n                <property name=\"position\">1</property>\n  "
  "            </packing>\n            </child>\n            <child>\n    "
  "          <object class=\"GtkStack\" id=\"stack\">\n                <pr"
  "operty name=\"visible\">True</property>\n                <property name"
  "=\"can_focus\">False</property>\n                <property name=\"trans"
  "ition_type\">slide-left-right</property>\n                <child>\n    "
  "              <object class=\"GtkGrid\" id=\"appearance\">\n           "
  "         <property name=\"visible\">True</property>\n                  "
  "  <property name=\"can_focus\">False</property>\n                    <p"
  "roperty name=\"margin_left\">12</property>\n                    <proper"
  "ty name=\"margin_right\">12</property>\n                    <property n"
  "ame=\"margin_top\">12</property>\n                    <property name=\""
  "margin_bottom\">12</property>\n                    <property name=\"row"
  "_spacing\">6</property>\n                    <property name=\"column_sp"
  "acing\">12</property>\n                    <property name=\"column_homo"
  "geneous\">True</property>\n                    <child>\n               "
  "       <object class=\"GtkScale\" id=\"opacity_slider\">\n             "
  "           <property name=\"visible\">True</property>\n                "
  "        <property name=\"can_focus\">True</property>\n                 "
  "       <property name=\"adjustment\">adjustment2</property>\n          "
  "              <property name=\"round_digits\">2</property>\n           "
  "             <property name=\"digits\">2</property>\n                  "
  "      <property name=\"value_pos\">right</property>\n                  "
  "    </object>\n                      <packing>\n                       "
  " <property name=\"left_attach\">1</property>\n                        <"
  "property name=\"top_attach\">6</property>\n                      </pack"
  "ing>\n                    </child>\n                    <child>\n      "
  "                <object class=\"GtkButton\" id=\"preview_button\">\n   "
  "                     <property name=\"label\" translatable=\"yes\">Show"
  " _Preview</property>\n                        <property name=\"visible\""
  ">True</property>\n                        <property name=\"can_focus\">"
  "True</property>\n                        <property name=\"receives_defa"
  "ult\">True</property>\n                        <property name=\"margin_"
  "top\">6</property>\n                        <property name=\"use_underl"
  "ine\">True</property>\n                      </object>\n               "
  "       <packing>\n                        <property name=\"left_attach\""
  ">1</property>\n                        <property name=\"top_attach\">9<"
  "/property>\n                      </packing>\n                    </chi"
  "ld>\n                    <child>\n                      <object class=\""
  "GtkBox\" id=\"hbox4\">\n                        <property name=\"visibl"
  "e\">True</property>\n                        <property name=\"can_focus"
  "\">False</property>\n                        <property name=\"spacing\""
  ">6</property>\n                        <child>\n                       "
  "   <object class=\"GtkLabel\" id=\"label4\">\n                         "
  "   <property name=\"visible\">True</property>\n                        "
  "    <property name=\"can_focus\">False</property>\n                    "
  "        <property name=\"label\" translatable=\"yes\">seconds</property"
  ">\n                            <property name=\"xalign\">0</property>\n"
  "                          </object>\n                          <packing"
  ">\n                            <property name=\"expand\">False</propert"
  "y>\n                            <property name=\"fill\">True</property>"
  "\n                            <property name=\"pack_type\">end</propert"
  "y>\n                            <property name=\"position\">0</property"
  ">\n                          </packing>\n                        </chil"
  "d>\n                        <child>\n                          <object "
  "class=\"GtkSpinButton\" id=\"expire_timeout_sbtn\">\n                  "
  "          <property name=\"visible\">True</property>\n                 "
  "           <property name=\"can_focus\">True</property>\n              "
  "              <property name=\"invisible_char\">\342\227\217</property>"
  "\n                            <property name=\"primary_icon_activatable"
  "\">False</property>\n                            <property name=\"secon"
  "dary_icon_activatable\">False</property>\n                            <"
  "property name=\"adjustment\">adjustment1</property>\n                  "
  "          <property name=\"numeric\">True</property>\n                 "
  "         </object>\n                          <packing>\n              "
  "              <property name=\"expand\">False</property>\n             "
  "               <property name=\"fill\">False</property>\n              "
  "              <property name=\"pack_type\">end</property>\n            "
  "                <property name=\"position\">1</property>\n             "
  "             </packing>\n                        </child>\n            "
  "          </object>\n                      <packing>\n                 "
  "       <property name=\"left_attach\">1</property>\n                   "
  "     <property name=\"top_attach\">7</property>\n                      "
  "</packing>\n                    </child>\n                    <child>\n"
  "                      <object class=\"GtkLabel\" id=\"label1\">\n      "
  "                  <property name=\"visible\">True</property>\n         "
  "               <property name=\"can_focus\">False</property>\n         "
  "               <property name=\"margin_left\">12</property>\n          "
  "              <property name=\"label\" translatable=\"yes\">_Disappear "
  "after</property>\n                        <property name=\"use_underlin"
  "e\">True</property>\n                        <property name=\"mnemonic_"
  "widget\">expire_timeout_sbtn</property>\n                        <prope"
  "rty name=\"xalign\">0</property>\n                      </object>\n    "
  "                  <packing>\n                        <property name=\"l"
  "eft_attach\">0</property>\n                        <property name=\"top"
  "_attach\">7</property>\n                      </packing>\n             "
  "       </child>\n                    <child>\n                      <ob"
  "ject class=\"GtkLabel\" id=\"label2\">\n                        <proper"
  "ty name=\"visible\">True</property>\n                        <property "
  "name=\"can_focus\">False</property>\n                        <property "
  "name=\"margin_left\">12</property>\n                        <property n"
  "ame=\"label\" translatable=\"yes\">_Opacity</property>\n               "
  "         <property name=\"use_underline\">True</property>\n            "
  "            <property name=\"xalign\">0</property>\n                   "
  "   </object>\n                      <packing>\n                        "
  "<property name=\"left_attach\">0</property>\n                        <p"
  "roperty name=\"top_attach\">6</property>\n                      </packi"
  "ng>\n                    </child>\n                    <child>\n       "
  "               <object class=\"GtkSwitch\" id=\"do_fadeout\">\n        "
  "                <property name=\"visible\">True</property>\n           "
  "             <property name=\"can_focus\">True</property>\n            "
  "            <property name=\"halign\">end</property>\n                 "
  "       <property name=\"valign\">center</property>\n                   "
  "   </object>\n                      <packing>\n                        "
  "<property name=\"left_attach\">1</property>\n                        <p"
  "roperty name=\"top_attach\">8</property>\n                      </packi"
  "ng>\n                    </child>\n                    <child>\n       "
  "               <object class=\"GtkLabel\" id=\"label13\">\n            "
  "            <property name=\"visible\">True</property>\n               "
  "         <property name=\"can_focus\">False</property>\n               "
  "         <property name=\"margin_left\">12</property>\n                "
  "        <property name=\"label\" translatable=\"yes\">Fade out</propert"
  "y>\n                        <property name=\"xalign\">0</property>\n   "
  "                   </object>\n                      <packing>\n        "
  "                <property name=\"left_attach\">0</property>\n          "
  "              <property name=\"top_attach\">8</property>\n             "
  "         </packing>\n                    </child>\n                    "
  "<child>\n                      <object class=\"GtkLabel\" id=\"label3\""
  ">\n                        <property name=\"visible\">True</property>\n"
  "                        <property name=\"can_focus\">False</property>\n"
  "                        <property name=\"margin_left\">12</property>\n "
  "                       <property name=\"label\" translatable=\"yes\">De"
  "fault _position</property>\n                        <property name=\"us"
  "e_underline\">True</property>\n                        <property name=\""
  "xalign\">0</property>\n                      </object>\n               "
  "       <packing>\n                        <property name=\"left_attach\""
  ">0</property>\n                        <property name=\"top_attach\">5<"
  "/property>\n                      </packing>\n                    </chi"
  "ld>\n                    <child>\n                      <object class=\""
  "GtkComboBox\" id=\"position_combo\">\n                        <property"
  " name=\"visible\">True</property>\n                        <property na"
  "me=\"can_focus\">False</property>\n                        <property na"
  "me=\"model\">model1</property>\n                        <child>\n      "
  "                    <object class=\"GtkCellRendererText\" id=\"renderer"
  "3\"/>\n                          <attributes>\n                        "
  "    <attribute name=\"text\">0</attribute>\n                          <"
  "/attributes>\n                        </child>\n                      <"
  "/object>\n                      <packing>\n                        <pro"
  "perty name=\"left_attach\">1</property>\n                        <prope"
  "rty name=\"top_attach\">5</property>\n                      </packing>\n"
  "                    </child>\n                    <child>\n            "
  "          <object class=\"GtkComboBox\" id=\"theme_combo\">\n          "
  "              <property name=\"name\">theme</property>\n               "
  "         <property name=\"visible\">True</property>\n                  "
  "      <property name=\"can_focus\">False</property>\n                  "
  "      <child>\n                          <object class=\"GtkCellRendere"
  "rText\" id=\"renderer1\"/>\n                          <attributes>\n   "
  "                         <attribute name=\"text\">0</attribute>\n      "
  "                    </attributes>\n                        </child>\n  "
  "                    </object>\n                      <packing>\n       "
  "                 <property name=\"left_attach\">1</property>\n         "
  "               <property name=\"top_attach\">4</property>\n            "
  "          </packing>\n                    </child>\n                   "
  " <child>\n                      <object class=\"GtkLabel\" id=\"label5\""
  ">\n                        <property name=\"visible\">True</property>\n"
  "                        <property name=\"can_focus\">False</property>\n"
  "                        <property name=\"margin_left\">12</property>\n "
  "                       <property name=\"label\" translatable=\"yes\">_T"
  "heme</property>\n                        <property name=\"use_underline"
  "\">True</property>\n                        <property name=\"xalign\">0"
  "</property>\n                      </object>\n                      <pa"
  "cking>\n                        <property name=\"left_attach\">0</prope"
  "rty>\n                        <property name=\"top_attach\">4</property"
  ">\n                      </packing>\n                    </child>\n    "
  "                <child>\n                      <object class=\"GtkLabel"
  "\" id=\"do_not_disturb_label\">\n                        <property name"
  "=\"visible\">True</property>\n                        <property name=\""
  "can_focus\">False</property>\n                        <property name=\""
  "margin_left\">12</property>\n                        <property name=\"h"
  "expand\">True</property>\n                        <property name=\"labe"
  "l\" translatable=\"yes\">Do not disturb</property>\n                   "
  "     <property name=\"xalign\">0</property>\n                      </ob"
  "ject>\n                      <packing>\n                        <proper"
  "ty name=\"left_attach\">0</property>\n                        <property"
  " name=\"top_attach\">1</property>\n                      </packing>\n  "
  "                  </child>\n                    <child>\n              "
  "        <object class=\"GtkSwitch\" id=\"do_not_disturb\">\n           "
  "             <property name=\"visible\">True</property>\n              "
  "          <property name=\"can_focus\">True</property>\n               "
  "         <property name=\"halign\">end</property>\n                    "
  "    <property name=\"valign\">center</property>\n                      "
  "</object>\n                      <packing>\n                        <pr"
  "operty name=\"left_attach\">1</property>\n                        <prop"
  "erty name=\"top_attach\">1</property>\n                      </packing>"
  "\n                    </child>\n                    <child>\n          "
  "            <object class=\"GtkLabel\">\n                        <prope"
  "rty name=\"visible\">True</property>\n                        <property"
  " name=\"can_focus\">False</property>\n                        <property"
  " name=\"label\" translatable=\"yes\">&lt;b&gt;Behavior&lt;/b&gt;</prope"
  "rty>\n                        <property name=\"use_markup\">True</prope"
  "rty>\n                        <property name=\"xalign\">0</property>\n "
  "                     </object>\n                      <packing>\n      "
  "                  <property name=\"left_attach\">0</property>\n        "
  "                <property name=\"top_attach\">0</property>\n           "
  "           </packing>\n                    </child>\n                  "
  "  <child>\n                      <object class=\"GtkLabel\">\n         "
  "               <property name=\"visible\">True</property>\n            "
  "            <property name=\"can_focus\">False</property>\n            "
  "            <property name=\"margin_top\">12</property>\n              "
  "          <property name=\"label\" translatable=\"yes\">&lt;b&gt;Appear"
  "ance&lt;/b&gt;</property>\n                        <property name=\"use"
  "_markup\">True</property>\n                        <property name=\"xal"
  "ign\">0</property>\n                      </object>\n                  "
  "    <packing>\n                        <property name=\"left_attach\">0"
  "</property>\n                        <property name=\"top_attach\">3</p"
  "roperty>\n                      </packing>\n                    </child"
  ">\n                    <child>\n                      <object class=\"G"
  "tkComboBox\" id=\"primary_monitor\">\n                        <property"
  " name=\"visible\">True</property>\n                        <property na"
  "me=\"can_focus\">False</property>\n                        <property na"
  "me=\"model\">liststore3</property>\n                        <property n"
  "ame=\"active_id\">1</property>\n                        <child>\n      "
  "                    <object class=\"GtkCellRendererText\" id=\"renderer"
  "5\"/>\n                          <attributes>\n                        "
  "    <attribute name=\"text\">0</attribute>\n                          <"
  "/attributes>\n                        </child>\n                      <"
  "/object>\n                      <packing>\n                        <pro"
  "perty name=\"left_attach\">1</property>\n                        <prope"
  "rty name=\"top_attach\">2</property>\n                      </packing>\n"
  "                    </child>\n                    <child>\n            "
  "          <object class=\"GtkLabel\" id=\"primary_label\">\n           "
  "             <property name=\"visible\">True</property>\n              "
  "          <property name=\"can_focus\">False</property>\n              "
  "          <property name=\"tooltip_text\" translatable=\"yes\">By defau"
  "lt the notification bubbles will be shown on the display on which the m"
  "ouse pointer is located.</property>\n                        <property "
  "name=\"margin_left\">12</property>\n                        <property n"
  "ame=\"label\" translatable=\"yes\">Show notifications on</property>\n  "
  "                      <property name=\"ellipsize\">end</property>\n    "
  "                    <property name=\"xalign\">0</property>\n           "
  "           </object>\n                      <packing>\n                "
  "        <property name=\"left_attach\">0</property>\n                  "
  "      <property name=\"top_attach\">2</property>\n                     "
  " </packing>\n                    </child>\n                    <child>\n"
  "                      <placeholder/>\n                    </child>\n   "
  "                 <child>\n                      <placeholder/>\n       "
  "             </child>\n                    <child>\n                   "
  "   <placeholder/>\n                    </child>\n                  </ob"
  "ject>\n                  <packing>\n                    <property name="
  "\"name\">general</property>\n                    <property name=\"title"
  "\" translatable=\"yes\">General</property>\n                  </packing"
  ">\n                </child>\n                <child>\n                 "
  " <object class=\"GtkGrid\" id=\"applications\">\n                    <p"
  "roperty name=\"visible\">True</property>\n                    <property"
  " name=\"can_focus\">False</property>\n                    <property nam"
  "e=\"margin_left\">12</property>\n                    <property name=\"m"
  "argin_right\">12</property>\n                    <property name=\"margi"
  "n_top\">12</property>\n                    <property name=\"margin_bott"
  "om\">12</property>\n                    <child>\n                      "
  "<object class=\"GtkLabel\" id=\"label9\">\n                        <pro"
  "perty name=\"visible\">True</property>\n                        <proper"
  "ty name=\"can_focus\">False</property>\n                        <proper"
  "ty name=\"margin_bottom\">6</property>\n                        <proper"
  "ty name=\"label\" translatable=\"yes\">&lt;b&gt;Show or block notificat"
  "ions per application&lt;/b&gt;</property>\n                        <pro"
  "perty name=\"use_markup\">True</property>\n                        <pro"
  "perty name=\"xalign\">0</property>\n                      </object>\n  "
  "                    <packing>\n                        <property name=\""
  "left_attach\">0</property>\n                        <property name=\"to"
  "p_attach\">0</property>\n                        <property name=\"width"
  "\">2</property>\n                      </packing>\n                    "
  "</child>\n                    <child>\n                      <object cl"
  "ass=\"GtkScrolledWindow\" id=\"known_applications_scrolled_window\">\n "
  "                       <property name=\"visible\">True</property>\n    "
  "                    <property name=\"can_focus\">True</property>\n     "
  "                   <property name=\"hexpand\">True</property>\n        "
  "                <property name=\"vexpand\">True</property>\n           "
  "             <property name=\"shadow_type\">in</property>\n            "
  "          </object>\n                      <packing>\n                 "
  "       <property name=\"left_attach\">0</property>\n                   "
  "     <property name=\"top_attach\">1</property>\n                      "
  "  <property name=\"width\">2</property>\n                      </packin"
  "g>\n                    </child>\n                  </object>\n        "
  "          <packing>\n                    <property name=\"name\">applic"
  "ations</property>\n                    <property name=\"title\" transla"
  "table=\"yes\">Applications</property>\n                    <property na"
  "me=\"position\">1</property>\n                  </packing>\n           "
  "     </child>\n                <child>\n                  <object class"
  "=\"GtkGrid\" id=\"log\">\n                    <property name=\"visible\""
  ">True</property>\n                    <property name=\"can_focus\">Fals"
  "e</property>\n                    <property name=\"margin_left\">12</pr"
  "operty>\n                    <property name=\"margin_right\">12</proper"
  "ty>\n                    <property name=\"margin_top\">12</property>\n "
  "                   <property name=\"margin_bottom\">12</property>\n    "
  "                <property name=\"row_spacing\">6</property>\n          "
  "          <property name=\"column_spacing\">12</property>\n            "
  "        <child>\n                      <object class=\"GtkLabel\" id=\""
  "log_level_label\">\n                        <property name=\"visible\">"
  "True</property>\n                        <property name=\"can_focus\">F"
  "alse</property>\n                        <property name=\"label\" trans"
  "latable=\"yes\">Log notifications</property>\n                        <"
  "property name=\"xalign\">0</property>\n                      </object>\n"
  "                      <packing>\n                        <property name"
  "=\"left_attach\">0</property>\n                        <property name=\""
  "top_attach\">0</property>\n                      </packing>\n          "
  "          </child>\n                    <child>\n                      "
  "<object class=\"GtkSwitch\" id=\"log_switch\">\n                       "
  " <property name=\"visible\">True</property>\n                        <p"
  "roperty name=\"can_focus\">True</property>\n                        <pr"
  "operty name=\"halign\">end</property>\n                        <propert"
  "y name=\"valign\">center</property>\n                      </object>\n "
  "                     <packing>\n                        <property name="
  "\"left_attach\">2</property>\n                        <property name=\""
  "top_attach\">0</property>\n                      </packing>\n          "
  "          </child>\n                    <child>\n                      "
  "<object class=\"GtkComboBox\" id=\"log_level\">\n                      "
  "  <property name=\"visible\">True</property>\n                        <"
  "property name=\"can_focus\">False</property>\n                        <"
  "property name=\"model\">liststore1</property>\n                        "
  "<property name=\"active_id\">1</property>\n                        <chi"
  "ld>\n                          <object class=\"GtkCellRendererText\" id"
  "=\"renderer2\"/>\n                          <attributes>\n             "
  "               <attribute name=\"text\">0</attribute>\n                "
  "          </attributes>\n                        </child>\n            "
  "          </object>\n                      <packing>\n                 "
  "       <property name=\"left_attach\">1</property>\n                   "
  "     <property name=\"top_attach\">0</property>\n                      "
  "</packing>\n                    </child>\n                    <child>\n"
  "                      <object class=\"GtkLabel\" id=\"log_level_apps_la"
  "bel\">\n                        <property name=\"visible\">True</proper"
  "ty>\n                        <property name=\"can_focus\">False</proper"
  "ty>\n                        <property name=\"label\" translatable=\"ye"
  "s\">Log applications</property>\n                        <property name"
  "=\"xalign\">0</property>\n                      </object>\n            "
  "          <packing>\n                        <property name=\"left_atta"
  "ch\">0</property>\n                        <property name=\"top_attach\""
  ">1</property>\n                      </packing>\n                    </"
  "child>\n                    <child>\n                      <object clas"
  "s=\"GtkBox\">\n                        <property name=\"visible\">True<"
  "/property>\n                        <property name=\"can_focus\">False<"
  "/property>\n                        <property name=\"hexpand\">True</pr"
  "operty>\n                        <property name=\"vexpand\">True</prope"
  "rty>\n                        <property name=\"orientation\">vertical</"
  "property>\n                        <child>\n                          <"
  "object class=\"GtkScrolledWindow\" id=\"log_scrolled_window\">\n       "
  "                     <property name=\"visible\">True</property>\n      "
  "                      <property name=\"can_focus\">True</property>\n   "
  "                         <property name=\"hexpand\">True</property>\n  "
  "                          <property name=\"vexpand\">True</property>\n "
  "                           <property name=\"shadow_type\">in</property>"
  "\n                          </object>\n                          <packi"
  "ng>\n                            <property name=\"expand\">False</prope"
  "rty>\n                            <property name=\"fill\">True</propert"
  "y>\n                            <property name=\"position\">0</property"
  ">\n                          </packing>\n                        </chil"
  "d>\n                        <child>\n                          <object "
  "class=\"GtkToolbar\" id=\"log_toolbar\">\n                            <"
  "property name=\"visible\">True</property>\n                            "
  "<property name=\"can_focus\">False</property>\n                        "
  "    <style>\n                              <class name=\"inline-toolbar"
  "\"/>\n                            </style>\n                          <"
  "/object>\n                          <packing>\n                        "
  "    <property name=\"expand\">False</property>\n                       "
  "     <property name=\"fill\">True</property>\n                         "
  "   <property name=\"position\">1</property>\n                          "
  "</packing>\n                        </child>\n                      </o"
  "bject>\n                      <packing>\n                        <prope"
  "rty name=\"left_attach\">0</property>\n                        <propert"
  "y name=\"top_attach\">2</property>\n                        <property n"
  "ame=\"width\">3</property>\n                      </packing>\n         "
  "           </child>\n                    <child>\n                     "
  " <object class=\"GtkComboBox\" id=\"log_level_apps\">\n                "
  "        <property name=\"visible\">True</property>\n                   "
  "     <property name=\"can_focus\">False</property>\n                   "
  "     <property name=\"model\">liststore2</property>\n                  "
  "      <property name=\"active_id\">0</property>\n                      "
  "  <child>\n                          <object class=\"GtkCellRendererTex"
  "t\" id=\"renderer4\"/>\n                          <attributes>\n       "
  "                     <attribute name=\"text\">0</attribute>\n          "
  "                </attributes>\n                        </child>\n      "
  "                </object>\n                      <packing>\n           "
  "             <property name=\"left_attach\">1</property>\n             "
  "           <property name=\"top_attach\">1</property>\n                "
  "      </packing>\n                    </child>\n                    <ch"
  "ild>\n                      <placeholder/>\n                    </child"
  ">\n                  </object>\n                  <packing>\n          "
  "          <property name=\"name\">log</property>\n                    <"
  "property name=\"title\" translatable=\"yes\">Log</property>\n          "
  "          <property name=\"position\">2</property>\n                  <"
  "/packing>\n                </child>\n              </object>\n         "
  "     <packing>\n                <property name=\"expand\">True</propert"
  "y>\n                <property name=\"fill\">True</property>\n          "
  "      <property name=\"position\">2</property>\n              </packing"
  ">\n            </child>\n          </object>\n          <packing>\n    "
  "        <property name=\"expand\">True</property>\n            <propert"
  "y name=\"fill\">True</property>\n            <property name=\"position\""
  ">1</property>\n          </packing>\n        </child>\n      </object>\n"
  "    </child>\n    <action-widgets>\n      <action-widget response=\"1\""
  ">close_btn</action-widget>\n    </action-widgets>\n  </object>\n</inter"
  "face>\n"
};

static const unsigned xfce4_notifyd_config_ui_length = 37399u;

