#XfceNotifyWindow {

    background-color: #4d4d4d;
    border-radius: 20px;
    border-color : #ffffff;
    border-style: outset;
    border-width: 2px;
    padding: 20px;
}

#XfceNotifyWindow:hover {
    border-width: 3px;
    border-color : #ffffff;
}

#XfceNotifyWindow GtkLabel,
#XfceNotifyWindow GtkImage {
    color: #ffffff;
    -GtkWidget-link-color:#a7a7a7;
}

#XfceNotifyWindow GtkLabel#summary {
    font : Bold;
}

#XfceNotifyWindow GtkButton {

    border-image: none;
    background-image: none;
    background-color : #202020;
    color: #ffffff;
}

#XfceNotifyWindow GtkButton:hover {

    border-image: none;
    background-image: none;
    background-color : #404040;
    color: #ffffff;
}
