/* this file is autogenerated -- do not edit */

#ifndef __xfce_notify_marshal_MARSHAL_H__
#define __xfce_notify_marshal_MARSHAL_H__

#include	<glib-object.h>

G_BEGIN_DECLS

/* VOID:UINT,UINT (./xfce4-notifyd/xfce-notify-marshal.list:1) */
extern void xfce_notify_marshal_VOID__UINT_UINT (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
                                                 const GValue *param_values,
                                                 gpointer      invocation_hint,
                                                 gpointer      marshal_data);

/* VOID:UINT,STRING (./xfce4-notifyd/xfce-notify-marshal.list:2) */
extern void xfce_notify_marshal_VOID__UINT_STRING (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
                                                   const GValue *param_values,
                                                   gpointer      invocation_hint,
                                                   gpointer      marshal_data);

G_END_DECLS

#endif /* __xfce_notify_marshal_MARSHAL_H__ */

